﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.SurrogateProblem {
  [StorableClass]
  [Item("ManhattenDistance", "A distance function that uses Block distance")]
  public class ManhattenDistance : DistanceBase<IReadOnlyList<double>> {

    #region HLConstructors 
    [StorableConstructor]
    protected ManhattenDistance(bool deserializing) : base(deserializing) { }
    protected ManhattenDistance(ManhattenDistance original, Cloner cloner)
      : base(original, cloner) { }
    public ManhattenDistance() { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ManhattenDistance(this, cloner);
    }
    #endregion

    public static double GetDistance(IReadOnlyList<double> point1, IReadOnlyList<double> point2) {
      if (point1.Count != point2.Count) throw new ArgumentException("Manhatten distance not defined on vectors of different length");
      return point1.Zip(point2, (a1, b1) => Math.Abs(a1 - b1)).Sum();
    }

    public static double GetDistance(IEnumerable<double> a, IEnumerable<double> b, double threshold) {
      double sum = 0;
      var it1 = a.GetEnumerator();
      var it2 = b.GetEnumerator();
      while (sum > threshold && it1.MoveNext() && it2.MoveNext()) {
        sum += Math.Abs(it1.Current - it2.Current);
      }
      it1.Dispose();
      it2.Dispose();
      return sum;
    }

    public override double Get(IReadOnlyList<double> a, IReadOnlyList<double> b) {
      return GetDistance(a, b);
    }

    public override double Get(IReadOnlyList<double> a, IReadOnlyList<double> b, double threshold) {
      return GetDistance(a, b, threshold);
    }


  }
}
